@@ -4,13 +4,11 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"os"
 	"path/filepath"
 	"runtime"
 	"strings"
 
-	"github.com/itchyny/go-flags"
 	"github.com/mattn/go-isatty"
 
 	"github.com/itchyny/gojq"
@@ -18,7 +16,7 @@ import (
 
 const name = "gojq"
 
-const version = "0.12.5"
+const version = "0.12.6"
 
 var revision = "HEAD"
 
@@ -72,12 +70,15 @@ type flagopts struct {
 	InputYAML     bool              `long:"yaml-input" description:"read input as YAML"`
 	FromFile      string            `short:"f" long:"from-file" description:"load query from file"`
 	ModulePaths   []string          `short:"L" description:"directory to search modules from"`
-	Args          map[string]string `long:"arg" description:"set variable to string value" count:"2" unquote:"false"`
-	ArgsJSON      map[string]string `long:"argjson" description:"set variable to JSON value" count:"2" unquote:"false"`
-	SlurpFile     map[string]string `long:"slurpfile" description:"set variable to the JSON contents of the file" count:"2" unquote:"false"`
-	RawFile       map[string]string `long:"rawfile" description:"set variable to the contents of the file" count:"2" unquote:"false"`
+	Arg           map[string]string `long:"arg" description:"set variable to string value"`
+	ArgJSON       map[string]string `long:"argjson" description:"set variable to JSON value"`
+	SlurpFile     map[string]string `long:"slurpfile" description:"set variable to the JSON contents of the file"`
+	RawFile       map[string]string `long:"rawfile" description:"set variable to the contents of the file"`
+	Args          []interface{}     `long:"args" positional:"" description:"consume remaining arguments as positional string values"`
+	JSONArgs      []interface{}     `long:"jsonargs" positional:"" description:"consume remaining arguments as positional JSON values"`
 	ExitStatus    bool              `short:"e" long:"exit-status" description:"exit 1 when the last value is false or null"`
 	Version       bool              `short:"v" long:"version" description:"print version"`
+	Help          bool              `short:"h" long:"help" description:"print this help"`
 }
 
 var addDefaultModulePaths = true
@@ -95,24 +96,25 @@ func (cli *cli) run(args []string) int {
 
 func (cli *cli) runInternal(args []string) (err error) {
 	var opts flagopts
-	args, err = flags.NewParser(
-		&opts, flags.HelpFlag|flags.PassDoubleDash,
-	).ParseArgs(adjustArgs(args))
+	args, err = parseFlags(args, &opts)
 	if err != nil {
-		if err, ok := err.(*flags.Error); ok && err.Type == flags.ErrHelp {
-			fmt.Fprintf(cli.outStream, `%[1]s - Go implementation of jq
+		return &flagParseError{err}
+	}
+	if opts.Help {
+		fmt.Fprintf(cli.outStream, `%[1]s - Go implementation of jq
 
 Version: %s (rev: %s/%s)
 
 Synopsis:
   %% echo '{"foo": 128}' | %[1]s '.foo'
 
+Usage:
+  %[1]s [OPTIONS]
+
 `,
-				name, version, revision, runtime.Version())
-			fmt.Fprintln(cli.outStream, err.Error())
-			return nil
-		}
-		return &flagParseError{err}
+			name, version, revision, runtime.Version())
+		fmt.Fprintln(cli.outStream, formatFlags(&opts))
+		return nil
 	}
 	if opts.Version {
 		fmt.Fprintf(cli.outStream, "%s %s (rev: %s/%s)\n", name, version, revision, runtime.Version())
@@ -125,10 +127,11 @@ Synopsis:
 	defer func(x bool) { noColor = x }(noColor)
 	if opts.OutputColor || opts.OutputMono {
 		noColor = opts.OutputMono
-	} else if os.Getenv("NO_COLOR") != "" {
+	} else if os.Getenv("NO_COLOR") != "" || os.Getenv("TERM") == "dumb" {
 		noColor = true
 	} else {
-		noColor = !isTTY(cli.outStream)
+		f, ok := cli.outStream.(interface{ Fd() uintptr })
+		noColor = !(ok && (isatty.IsTerminal(f.Fd()) || isatty.IsCygwinTerminal(f.Fd())))
 	}
 	if !noColor {
 		if colors := os.Getenv("GOJQ_COLORS"); colors != "" {
@@ -149,11 +152,11 @@ Synopsis:
 	}
 	cli.inputRaw, cli.inputSlurp, cli.inputStream, cli.inputYAML =
 		opts.InputRaw, opts.InputSlurp, opts.InputStream, opts.InputYAML
-	for k, v := range opts.Args {
+	for k, v := range opts.Arg {
 		cli.argnames = append(cli.argnames, "$"+k)
 		cli.argvalues = append(cli.argvalues, v)
 	}
-	for k, v := range opts.ArgsJSON {
+	for k, v := range opts.ArgJSON {
 		val, _ := newJSONInputIter(strings.NewReader(v), "$"+k).Next()
 		if err, ok := val.(error); ok {
 			return err
@@ -170,7 +173,7 @@ Synopsis:
 		cli.argvalues = append(cli.argvalues, val)
 	}
 	for k, v := range opts.RawFile {
-		val, err := ioutil.ReadFile(v)
+		val, err := os.ReadFile(v)
 		if err != nil {
 			return err
 		}
@@ -181,11 +184,28 @@ Synopsis:
 	for i, name := range cli.argnames {
 		named[name[1:]] = cli.argvalues[i]
 	}
+	positional := opts.Args
+	for i, v := range opts.JSONArgs {
+		if v != nil {
+			val, _ := newJSONInputIter(strings.NewReader(v.(string)), "--jsonargs").Next()
+			if err, ok := val.(error); ok {
+				return err
+			}
+			if i < len(positional) {
+				positional[i] = val
+			} else {
+				positional = append(positional, val)
+			}
+		}
+	}
 	cli.argnames = append(cli.argnames, "$ARGS")
-	cli.argvalues = append(cli.argvalues, map[string]interface{}{"named": named})
+	cli.argvalues = append(cli.argvalues, map[string]interface{}{
+		"named":      named,
+		"positional": positional,
+	})
 	var arg, fname string
 	if opts.FromFile != "" {
-		src, err := ioutil.ReadFile(opts.FromFile)
+		src, err := os.ReadFile(opts.FromFile)
 		if err != nil {
 			return err
 		}
@@ -199,7 +219,7 @@ Synopsis:
 	if opts.ExitStatus {
 		cli.exitCodeError = &exitCodeError{exitCodeNoValueErr}
 		defer func() {
-			if er, ok := err.(interface{ ExitCode() int }); !ok || er.ExitCode() == 0 {
+			if _, ok := err.(interface{ ExitCode() int }); !ok {
 				err = cli.exitCodeError
 			}
 		}()
@@ -221,12 +241,14 @@ Synopsis:
 		gojq.WithFunction("debug", 0, 0, cli.funcDebug),
 		gojq.WithFunction("stderr", 0, 0, cli.funcStderr),
 		gojq.WithFunction("input_filename", 0, 0,
-			func(interface{}, []interface{}) interface{} {
-				if fname := iter.Name(); fname != "" {
-					return fname
+			func(iter inputIter) func(interface{}, []interface{}) interface{} {
+				return func(interface{}, []interface{}) interface{} {
+					if fname := iter.Name(); fname != "" && (len(args) > 0 || !opts.InputNull) {
+						return fname
+					}
+					return nil
 				}
-				return nil
-			},
+			}(iter),
 		),
 		gojq.WithInputIter(iter),
 	)
@@ -254,30 +276,6 @@ Synopsis:
 	return cli.process(iter, code)
 }
 
-// insert double dash to recognize query with leading hyphen
-func adjustArgs(args []string) []string {
-	for i := len(args) - 1; i >= 0; i-- {
-		if arg := args[i]; strings.HasPrefix(arg, "-") {
-			if strings.ContainsAny(arg, " .|,$?+*/%=!<>@()[]") {
-				j, hasDoubleDash := i, false
-				for ; j >= 0; j-- {
-					if args[j] == "--" {
-						hasDoubleDash = true
-						break
-					}
-				}
-				if !hasDoubleDash {
-					args = append(args, "")
-					copy(args[i+1:], args[i:])
-					args[i] = "--"
-				}
-			}
-			break
-		}
-	}
-	return args
-}
-
 func listDefaultModulePaths() []string {
 	modulePaths := []string{"", "../lib/gojq", "lib"}
 	if executable, err := os.Executable(); err == nil {
@@ -439,12 +437,3 @@ func (cli *cli) printError(err error) {
 		}
 	}
 }
-
-// isTTY attempts to determine whether an output is a TTY.
-func isTTY(w io.Writer) bool {
-	if os.Getenv("TERM") == "dumb" {
-		return false
-	}
-	f, ok := w.(interface{ Fd() uintptr })
-	return ok && (isatty.IsTerminal(f.Fd()) || isatty.IsCygwinTerminal(f.Fd()))
-}
