@@ -3,8 +3,8 @@ package gojq_test
 import (
 	"context"
 	"fmt"
-	"io/ioutil"
 	"log"
+	"os"
 	"reflect"
 	"sync"
 	"testing"
@@ -105,7 +105,7 @@ func ExampleCode_RunWithContext() {
 }
 
 func TestCodeCompile_OptimizeConstants(t *testing.T) {
-	query, err := gojq.Parse("[1,{foo:2},[3]]")
+	query, err := gojq.Parse(`[1,{foo:2,"bar":3},[4]]`)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -123,15 +123,15 @@ func TestCodeCompile_OptimizeConstants(t *testing.T) {
 			break
 		}
 		if expected := []interface{}{
-			1, map[string]interface{}{"foo": 2}, []interface{}{3},
+			1, map[string]interface{}{"foo": 2, "bar": 3}, []interface{}{4},
 		}; !reflect.DeepEqual(got, expected) {
 			t.Errorf("expected: %v, got: %v", expected, got)
 		}
 	}
 }
 
-func TestCodeCompile_OptimizeTailRec_Range(t *testing.T) {
-	query, err := gojq.Parse("range(10)")
+func TestCodeCompile_OptimizeTailRec_While(t *testing.T) {
+	query, err := gojq.Parse("0 | while(. < 10; . + 1)")
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -140,11 +140,11 @@ func TestCodeCompile_OptimizeTailRec_Range(t *testing.T) {
 		t.Fatal(err)
 	}
 	codes := reflect.ValueOf(code).Elem().FieldByName("codes")
-	if got, expected := codes.Len(), 141; expected != got {
+	if got, expected := codes.Len(), 48; expected != got {
 		t.Errorf("expected: %v, got: %v", expected, got)
 	}
-	op1 := codes.Index(1).Elem().FieldByName("op")
-	op2 := codes.Index(83).Elem().FieldByName("op") // test jump of _while
+	op1 := codes.Index(2).Elem().FieldByName("op")
+	op2 := codes.Index(21).Elem().FieldByName("op") // test jump of call _while
 	if got, expected := *(*int)(unsafe.Pointer(op2.UnsafeAddr())),
 		*(*int)(unsafe.Pointer(op1.UnsafeAddr())); expected != got {
 		t.Errorf("expected: %v, got: %v", expected, got)
@@ -251,7 +251,7 @@ func TestCodeRun_Race(t *testing.T) {
 }
 
 func BenchmarkCompile(b *testing.B) {
-	cnt, err := ioutil.ReadFile("builtin.jq")
+	cnt, err := os.ReadFile("builtin.jq")
 	if err != nil {
 		b.Fatal(err)
 	}
